import { defineStore } from "pinia";
import { WorkflowService } from "@/client/workflow";
import type { WorkflowVersionReduced } from "@/client/workflow";
import type { WorkflowOut } from "@/client/workflow";
import dayjs from "dayjs";

export const useWorkflowStore = defineStore({
  id: "workflows",
  state: () =>
    ({
      workflows: [],
    } as {
      workflows: WorkflowOut[];
    }),
  getters: {
    latestVersion(): (
      workflowId: string
    ) => WorkflowVersionReduced | undefined {
      return (workflowId) => {
        const workflow = this.workflows.find(
          (w) => workflowId == w.workflow_id
        );
        if (workflow == null || workflow.versions.length == 0) {
          return undefined;
        }
        const vs = [...workflow.versions];
        vs.sort((a, b) =>
          dayjs(a.created_at).isBefore(b.created_at) ? -1 : 1
        );
        return vs[vs.length - 1];
      };
    },
  },
  actions: {
    fetchWorkflows(
      onFulfilled:
        | ((workflows: WorkflowOut[]) => void)
        | null
        | undefined = null,
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      onRejected: ((reason: any) => void) | null | undefined = null,
      onFinally: (() => void) | null | undefined = null
    ) {
      WorkflowService.workflowListWorkflows()
        .then((workflows) => {
          this.workflows = workflows;
          onFulfilled?.(workflows);
        })
        .catch(onRejected)
        .finally(onFinally);
    },
  },
});
